from rest_framework.routers import DefaultRouter
from rest_framework.viewsets import ViewSet
from django.apps import apps
from importlib import import_module
import os
from django.conf import settings

def get_mainapp():
    settings_app = str(settings.SETTINGS_MODULE).split('.')[0]
    return settings_app
    
def getRouters():
    settings_app = get_mainapp()
    if settings_app:
        main_app = import_module(f'{settings_app}.routers')
        return main_app.getRouters()
    
def get_core_urls_path():
    settings_app = get_mainapp()
    if settings_app:
        main_app_urls = os.path.join(settings_app,'urls.py')
        return main_app_urls
      

class ViewSetsHelper:
    def __init__(self,app_name,model_name) -> None:
        self.app_name = app_name
        self.model_name = model_name
        pass
        # self.app_name = app_name
        # self.model_name = model_name
        
    def find_viewsets_for_model(self):
        app_config = apps.get_app_config(self.app_name)
        model = app_config.get_model(str(self.model_name))
        
        registered_viewsets = []
        serializers_for_model = {}  # Define serializers_for_model dictionary

        router = getRouters()
        # Iterate over the registered viewsets in the router
        for prefix, viewset, basename in router.registry:
            if hasattr(viewset, 'queryset') and viewset.queryset.model == model:
                registered_viewsets.append((prefix, viewset, basename))

                # Check if the viewset has a serializer class
                if hasattr(viewset, 'serializer_class'):
                  
                    serializer_class = viewset.serializer_class
                    serializers_for_model = {
                        viewset.__name__ : serializer_class
                    }
                    
        
        return registered_viewsets
    
    
    




# from abc import ABC, abstractmethod
from django.apps import apps
from .supports.base_exists import BaseHelper


class BaseTask:
    def __init__(self, app_name, task_type):
        self.app_name = app_name
        self.type = task_type
        self.models_list = self.extract_models()

    def run(self):
        if isinstance(self.type, str):
            for model in self.models_list:
                task = BaseHelper(self.app_name, self.type, model.__name__)
                task.run()
        elif isinstance(self.type, dict):
            for key, items in self.type.items():
                for item in items:
                    for model in self.models_list:
                        task = BaseHelper(self.app_name, item, model.__name__, key)
                        task.run()

    def extract_models(self):
        app_config = apps.get_app_config(self.app_name)
        return list(app_config.get_models())



    
        


from django.apps import apps
import os

def get_project_module_name():
    # Get the directory of the script
    script_dir = os.path.dirname(os.path.realpath(__file__))
    # Navigate up until reaching the directory containing manage.py
    while not os.path.exists(os.path.join(script_dir, 'manage.py')):
        script_dir = os.path.dirname(script_dir)
    # Extract the project module name from the directory path
    project_module_name = script_dir
    return project_module_name

def get_custom_apps():
    custom_apps = []

    # Get the project's module name
    project_module_name = get_project_module_name()

    # Iterate over all installed apps
    for app_config in apps.get_app_configs():
        app_module = app_config.module
        # print(app_config.label)
        app_module_name = app_module.__name__
        if os.path.exists(os.path.join(project_module_name,app_module_name)):
            custom_apps.append(app_config.label)

    return custom_apps


from setuptools import setup, find_packages
import os

# Read the content of your README file
with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='drfapigenerator',
    version='3.0.6',
    include_package_data=True,
    
    install_requires=[
        'django',
        'djangorestframework',
        'djangorestframework-simplejwt'
    ],
    entry_points={
        'console_scripts': [
            # Define any command line scripts here
        ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Framework :: Django',
        'Framework :: Django :: 3.2',  # Specify the appropriate Django version
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',

    description="A Django app for automating RESTful API development using Django Rest Framework (DRF), simplifying CRUD operations for Django models.",
    long_description = long_description,
    long_description_content_type='text/markdown',
    author='Manoj Kumar Das',
    author_email='manojdas.py@gmail.com',
    url='https://github.com/mddas2/drfapigenerator',
    # packages=['drfapigenerator'],

    package_data={
        'drfapigenerator.management.commands.data.routers': ['*.txt'],
        'drfapigenerator.management.commands.data.serializers': ['*.txt'],
        'drfapigenerator.management.commands.data.utilities': ['*.txt'],
        'drfapigenerator.management.commands.data.viewsets': ['*.txt'],
    },
    
)






from .base_tasks import BaseTask
from .supports.helper.helper import get_core_urls_path

class AutoApi:
    tasks = [{'utilities': ['permissions', 'pagination', 'importbase']}, 'serializers', 'viewsets', 'routers']

    def __init__(self, app_name):
        self.app_name = app_name
        self.main_project_urls_path = get_core_urls_path()

    def make_apis(self):
        for task in self.tasks:
            task_obj = BaseTask(self.app_name, task)
            task_obj.run()

    def app_router_register(self):
        print("registering router")
        app_router_layout = f"""\n#*********This is {self.app_name} router registered by autoapi*********\nfrom {self.app_name}.routers.routers import router as {self.app_name}_router\nurlpatterns.append(path('api/',include({self.app_name}_router.urls)))"""
        with open(self.main_project_urls_path,'a') as main_project_urls_obj:
            main_project_urls_obj.write(app_router_layout)

        return True
    
    def make_app_router(self):
        matching_import_router = f"{self.app_name}.routers"
        is_router_registered = False
        with open(self.main_project_urls_path,'r') as urls_file_obj:
            urls_data = urls_file_obj.readlines()
            for item in urls_data:
                if matching_import_router in item:
                    is_router_registered = True
        
        if not is_router_registered:
            self.app_router_register()

                
            


        


    


            

    